//: DBStatus.java

package com.hp.turboimage;

/** The DBStatus class represents the TurboIMAGE status
 *  array.<p>
 *
 * @version 2.2 July 1998
 * @Author  Mike Yawn
 */
public class DBStatus implements Cloneable, java.io.Serializable {

   /** The status array.  Directly accessible to members
     * of the TurboIMAGE package; all others (programs
     * written to use the library) should use the
     * access methods provided.
     */
   private short halfword[] = new short[10];

   /** Returns the status of the last TurboIMAGE call.  */
   public int getStatus()        { return halfword[0]; }

   /** Returns the number of locks obtained or released.  Only
     * meaningful following lock() or unlock() operations. */
   public int getNumLocks()      { return halfword[1]; }

   /** Returns the length of the current entry.  */
   public int getEntryLength()   { return halfword[1]; }

   /** Returns the user class.  Only meaningful after open() call */
   public int getUserClass()     { return halfword[1]; }

   /** Returns 0 if a database lock is being held, or 1 if
     * a dataset or data entry lock is being held */
   public int getLockGranularity()  { return halfword[2]; }

   /** Returns the size (in halfwords) of the DBG area.  Only
     * meaningful after an open() call */
   public int getDBGSize()       { return halfword[2]; }

   /** Returns the size (in halfwords) of the DBU area.  Only
     * meaningful after an open() call */
   public int getDBUSize()       { return halfword[3]; }

   /** Returns the record number of the current entry */
   public int   getCurrentRecordNum(){
      return returnInt(halfword[2], halfword[3]); }

   /** Returns the number of synonyms for the current entry */
   public int   getNumSynonyms(){
      return returnInt(halfword[4], halfword[5]); }

   /** Returns the record number of the next record in the chain */
   public int   getNextRecordNum(){
      return returnInt(halfword[8], halfword[9]); }

   /** Returns the record number of the previous record in the chain */
   public int   getPrevRecordNum(){
      return returnInt(halfword[6], halfword[7]); }

   /** Returns the record number of the first record in the chain */
   public int   getFirstEntryNum(){
      return returnInt(halfword[8], halfword[9]); }

   /** Returns the record number of the last record in the chain */
   public int   getLastEntryNum(){
      return returnInt(halfword[6], halfword[7]); }

   /** Returns the number of entries in the chain */
   public int   getNumChainEntries(){
      return returnInt(halfword[4], halfword[5]); }

   /** Sets a value in the status array.  Only callable by the
     * package code.
     */
   void setHalfword(int i, short s) { halfword[i] = s; };

   /** Used to assemble two halfword values into an integer */
   private int returnInt(short hi, short lo){
      // return an integer from two halfwords
      return (hi << 16) + lo;
   }

   /** A main method allows this class to be tested in isolation.
     * It will never be invoked during normal usage. */
   public static void main(String[] args){

      DBStatus s = new DBStatus();
      s.halfword[0] = 111;
      s.halfword[1] = 222;
      s.halfword[2] = 5;
      s.halfword[3] = 5764;

      System.out.println("Status    (111) = " + s.getStatus());
      System.out.println("Locks     (222) = " + s.getNumLocks());
      System.out.println("CurRec (333444) = " + s.getCurrentRecordNum());
   }
}

