//: Database.java

    /*
              TurboIMAGE Class Library for Java/iX
              Copyright (C) 1997, 1998 by Mike Yawn
              Comments to:   myawn@cup.hp.com
      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
    */

package com.hp.turboimage;

import java.rmi.Remote;
import java.rmi.RemoteException;

/** This is the interface which defines a TurboIMAGE database.
 *  The primary reason for defining the interface separately from
 *  the Database implementation class is to meet the requirements
 *  of Remote Method Invocation.  Methods are not documented here;
 *  see the Database class.
 *
 * @version 2.0 (RMI enabled) January 1998
 * @author  Mike Yawn
 */
public interface Database extends Remote {

// Set database attributes
   public void setPassword(String pass) throws RemoteException;
   public void setName(String dbRootFile) throws RemoteException;

// Get database attributes
   public DBStatus getStatus() throws RemoteException;
   public byte[]   getName()   throws RemoteException;

// Open the database
   public void open(int mode) throws IMAGEException, RemoteException;
   public void openSharedModify() throws IMAGEException, RemoteException;
   public void openSemiExclusiveUpdate() throws IMAGEException, RemoteException;
   public void openSemiExclusiveUpdateAllowRead() 
               throws IMAGEException, RemoteException;
   public void openExclusiveModify() throws IMAGEException, RemoteException;
   public void openSemiExclusiveModifyAllowRead() 
               throws IMAGEException, RemoteException;
   public void openSharedRead() throws IMAGEException, RemoteException;
   public void openSemiExclusiveRead() throws IMAGEException, RemoteException;
   public void openSemiExclusiveReadAllowUpdate() 
               throws IMAGEException, RemoteException;
   public void openSemiExclusiveReadAllowModify() 
               throws IMAGEException, RemoteException;
   public void openSemiExclusiveReadAllowReadOnly() 
               throws IMAGEException, RemoteException;
   public void openExclusiveRead() throws IMAGEException, RemoteException;
   public void openSemiExclusiveReadOnly() 
               throws IMAGEException, RemoteException;

// Close the database
   public void close() throws IMAGEException, RemoteException;
 
// Database locking
   public boolean lock() throws IMAGEException, RemoteException;
   public boolean attemptLock() throws IMAGEException, RemoteException;
   public boolean unlock() throws IMAGEException, RemoteException;

}

