//: DatabaseInfoImpl.java

package com.hp.turboimage;

import java.rmi.RemoteException;

/** This is an abstract root class representing the data returned
 *  by calls to DBINFO.
 *
 * @version 2.0 (RMI enabled) January 1998
 * @author  Mike Yawn
 * @see  ItemInfo
 * @see  ItemList
 * @see  SetInfo
 * @see  SetList
 */
public abstract class DatabaseInfoImpl implements DatabaseInfo {

   protected byte[] buffer = new byte[1023];  // is this big enough?

   /** Request information using item number as key 
     * @param db     a Database object
     * @param item   The item or set number 
     * @param status a DBStatus object
     * @exception IMAGEException A TurboIMAGE error occured
     */
   public abstract void getInfo(Database db, int item, DBStatus status)
                        throws IMAGEException, RemoteException;

   /** Request information using item or set name as key  
     * @param db      a Database object
     * @param item    item or set name
     * @param status  a DBStatus object
     * @exception IMAGEException A TurboIMAGE error occured
     */
   public abstract void getInfo(Database db, String item, DBStatus status)
                        throws IMAGEException, RemoteException;

   public native void DBINFO (byte[] basename, Object qualifier,
                         int mode, DBStatus status, byte[] infobuf); 
}

