//: DatabaseImpl.c
#include "com_hp_turboimage_DatabaseImpl.h"

//*************************************************************
//********* TurboIMAGE Dataset implementation *****************
//*************************************************************

/* DBGET  All modes except 4*/
void Java_com_hp_turboimage_DatasetImpl_DBGET( JNIEnv    *env,
                                           jobject    this,
                                           jbyteArray j_basename,
                                           jstring    j_dataset,
                                           jint       j_mode,
                                           jobject    j_status,
                                           jstring    j_list,
                                           jbyteArray j_buffer,
                                           jobject    j_argument) {

   // Get JNI fields into C types
   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   const char *dataset = (*env)->GetStringUTFChars(env, j_dataset, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   const char    *list = (*env)->GetStringUTFChars(env, j_list, 0);
   jbyte       *buffer = (*env)->GetByteArrayElements(env, j_buffer, 0);
   short          mode = (short) j_mode;
   jmethodID  getValue;

   jint    arg_i;  // holds argument if it's an integer
   char   *arg_s;  // holds argument if it's a string
   void   *arg;    // points to variable holding argument

   // Get known classes (should move this to an initialization routine
   // so it only gets done once!)
   jclass jlstring = (*env)->FindClass(env,"java/lang/String");
   jclass jlint    = (*env)->FindClass(env,"java/lang/Integer");
 
   // Get argument info a field of correct type
   if ((*env)->IsInstanceOf(env, j_argument, jlstring)) {
      // A String was passed
      arg_s = (char *)(*env)->GetStringUTFChars(env, (jstring) j_argument, 0);
      arg   = arg_s; // was (void *) &arg_s;
   } else if ((*env)->IsInstanceOf(env, j_argument, jlint)) {
      // An Integer was passed
      getValue = (*env)->GetMethodID(env, jlint, "intValue", "I");
      arg_i    = (*env)->CallIntMethod(env, j_argument, getValue);
      arg      = (void *) &arg_i;
   } else {
      // should throw exception here
      printf("unexpected type for argument to DBGET\n");
      return;
   }

   dbget(base, dataset, &mode, a_halfword, list, buffer, arg);

   if (a_halfword[0] < 0){
      ThrowException(env, "DBGET", a_halfword);
      return;
   }

   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseStringUTFChars(env, j_dataset, dataset);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);
   (*env)->ReleaseStringUTFChars(env, j_list, list);
   (*env)->ReleaseByteArrayElements(env, j_buffer, buffer, 0);
   if ((*env)->IsInstanceOf(env, j_argument, jlstring)) {
      (*env)->ReleaseStringUTFChars(env, j_argument, arg_s);
   }
}

void Java_com_hp_turboimage_DatasetImpl_DBFIND(JNIEnv    *env,
                                           jobject    this,
                                           jbyteArray j_basename,
                                           jstring    j_dataset,
                                           jint       j_mode,
                                           jobject    j_status,
                                           jstring    j_item,
                                           jobject    j_argument){

  // Get JNI fields into C types
  jclass jlstring = (*env)->FindClass(env,"java/lang/String");
  jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
  const char *dataset = (*env)->GetStringUTFChars(env, j_dataset, 0);
  jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
  jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
  jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
  jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
  const char     *itm = (*env)->GetStringUTFChars(env, j_item, 0);
  const char     *arg;
  short mode = (short) j_mode;

  if ((*env)->IsInstanceOf(env, j_argument, jlstring)) {
     arg = (*env)->GetStringUTFChars(env, j_argument, 0);
  } else {
     printf("DBFIND can't handle non-character search items yet\n");
     return;
  }

  dbfind(base, dataset, &mode, a_halfword, itm, arg);

  if (a_halfword[0] < 0){
     ThrowException(env, "DBFIND", a_halfword);
     return;
  }

  (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
  (*env)->ReleaseStringUTFChars(env, j_dataset, dataset);
  (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);
  if ((*env)->IsInstanceOf(env, j_argument, jlstring)) {
     (*env)->ReleaseStringUTFChars(env, j_item, itm);
  }
  (*env)->ReleaseStringUTFChars(env, j_argument, arg);
}

void Java_com_hp_turboimage_DatasetImpl_DBPUT(JNIEnv    *env,
                                          jobject    this,
                                          jbyteArray j_basename,
                                          jstring    j_dataset,
                                          jint       j_mode,
                                          jobject    j_status,
                                          jstring    j_list,
                                          jbyteArray j_buffer) {

   // Get JNI fields into C types
   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   const char *dataset = (*env)->GetStringUTFChars(env, j_dataset, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   const char    *list = (*env)->GetStringUTFChars(env, j_list, 0);
   jbyte       *buffer = (*env)->GetByteArrayElements(env, j_buffer, 0);
   short mode = (short) j_mode;

   dbput(base, dataset, &mode, a_halfword, list, buffer);

   if (a_halfword[0] < 0){
      ThrowException(env, "DBPUT", a_halfword);
      return;
   }

   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseStringUTFChars(env, j_dataset, dataset);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);
   (*env)->ReleaseStringUTFChars(env, j_list, list);
   (*env)->ReleaseByteArrayElements(env, j_buffer, buffer, 0);

}

void Java_com_hp_turboimage_DatasetImpl_DBUPDATE(JNIEnv    *env,
                                             jobject    this,
                                             jbyteArray j_basename,
                                             jstring    j_dataset,
                                             jint       j_mode,
                                             jobject    j_status,
                                             jstring    j_list,
                                             jbyteArray j_buffer) {

   // Get JNI fields into C types
   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   const char *dataset = (*env)->GetStringUTFChars(env, j_dataset, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   const char    *list = (*env)->GetStringUTFChars(env, j_list, 0);
   jbyte       *buffer = (*env)->GetByteArrayElements(env, j_buffer, 0);
   short mode = (short) j_mode;

   dbupdate(base, dataset, &mode, a_halfword, list, buffer);

   if (a_halfword[0] < 0){
      ThrowException(env, "DBUPDATE", a_halfword);
      return;
   }

   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseStringUTFChars(env, j_dataset, dataset);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);
   (*env)->ReleaseStringUTFChars(env, j_list, list);
   (*env)->ReleaseByteArrayElements(env, j_buffer, buffer, 0);

}

void Java_com_hp_turboimage_DatasetImpl_DBDELETE(JNIEnv    *env,
                                             jobject    this,
                                             jbyteArray j_basename,
                                             jstring    j_dataset,
                                             jint       j_mode,
                                             jobject    j_status) {

   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   const char *dataset = (*env)->GetStringUTFChars(env, j_dataset, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   short mode = (short) j_mode;

   dbdelete(base, dataset, &mode, a_halfword);

   if (a_halfword[0] < 0){
      ThrowException(env, "DBDELETE", a_halfword);
      return;
   }

   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseStringUTFChars(env, j_dataset, dataset);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);

}

void Java_com_hp_turboimage_DatasetImpl_DBLOCK(JNIEnv    *env,
                                           jobject    this,
                                           jbyteArray j_basename,
                                           jobject    j_qualifier,
                                           jint       j_mode,
                                           jobject    j_status) {

   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   short mode = (short) j_mode;


   if ((mode == 3) || (mode == 4)) {
      // j_qualifier is a String (dataset lock)
      // qualif should be set to the string contents
      const char* qualif = (*env)->GetStringUTFChars(env, j_qualifier, 0);
      dblock(base, qualif, &mode, a_halfword);
      (*env)->ReleaseStringUTFChars(env, j_qualifier, qualif);
   } else {  // qualifier is a LockDescriptor object (item lock)
      // j_qualifier is a LockDescriptor object 
      // qualif should be set to the contents of the buffer field
      jclass     ldclass = (*env)->GetObjectClass(env, j_qualifier);
      jfieldID      pbuf = (*env)->GetFieldID(env, ldclass, "buffer", "[B");
      jobject       abuf = (*env)->GetObjectField(env, j_qualifier, pbuf);
      jbyte      *qualif = (*env)->GetByteArrayElements(env, abuf, 0);
      dblock (base, qualif, &mode, a_halfword);
      (*env)->ReleaseByteArrayElements(env, abuf , qualif, 0);
   }

   if (a_halfword[0] < 0){
      ThrowException(env, "DBLOCK", a_halfword);
      return;
   }

   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);

}

void Java_com_hp_turboimage_DatasetImpl_DBUNLOCK(JNIEnv    *env,
                                             jobject    this,
                                             jbyteArray j_basename,
                                             jstring    j_qualifier,
                                             jint       j_mode,
                                             jobject    j_status) {

   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   const char  *qualif = (*env)->GetStringUTFChars(env, j_qualifier, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   short mode = (short) j_mode;

   dbunlock(base, qualif, &mode, a_halfword);

   if (a_halfword[0] < 0){
      ThrowException(env, "DBUNLOCK", a_halfword);
      return;
   }

   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseStringUTFChars(env, j_qualifier, qualif);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);

}

/* DBCLOSE */
JNIEXPORT void JNICALL 
Java_com_hp_turboimage_DatasetImpl_DBCLOSE (JNIEnv    *env, 
                                         jobject    this, 
                                         jbyteArray j_basename, 
                                         jstring    j_dataset, 
                                         jint       j_mode, 
                                         jobject    j_status) {

    // Get JNI parms into C types
   jbyte         *base = (*env)->GetByteArrayElements(env, j_basename, 0);
   const char *dataset = (*env)->GetStringUTFChars(env, j_dataset, 0);
   jclass   c_DBStatus = (*env)->GetObjectClass(env, j_status);
   jfieldID f_halfword = (*env)->GetFieldID(env, c_DBStatus, "halfword", "[S");
   jobject  p_halfword = (*env)->GetObjectField(env, j_status, f_halfword);
   jshort  *a_halfword = (*env)->GetShortArrayElements(env, p_halfword, 0);
   jshort         mode = j_mode;

    dbclose(base, dataset, &mode, a_halfword);

    if(a_halfword[0] < 0) {
       ThrowException(env, "DBCLOSE", a_halfword);
    }

   (*env)->ReleaseStringUTFChars(env, j_dataset, dataset); 
   (*env)->ReleaseByteArrayElements(env, j_basename, base, 0);
   (*env)->ReleaseShortArrayElements(env, p_halfword, a_halfword, 0);
}
