//: Item.java

package com.hp.turboimage;

import java.lang.reflect.Method;

   /** Class Item represents a data item in the
     * database buffer.   An object of class Item can
     * be obtained by calling Entry.getItem(), but in most
     * cases, when all you are interested in is the value
     * of the item, you can use Entry.getValue(itemname)
     * and Entry.setValue(itemname) to access the item.
     *
     * Note that for compound items, the index values
     * begin at zero, even though TurboIMAGE subitems
     * begin with one.  
     * 
     * @version 2.2 July 1998
     * @author Mike Yawn
     * @see Entry#getItem()
     * @see Entry#getValue()
     * @see Entry#setValue()
     * 
     */

public class Item implements Cloneable, java.io.Serializable {
      // Following fields have package access, rather than private --
      // Item was originally an inner class of Entry (moved since
      // RMI doesn't yet handle inner classes properly). Class Entry
      // needs free access to these attributes.
      Object[] value;      // the value of the item
      int      type;       // As defined in class Entry
      int      length;     // Length of item in IMAGE buffer
      int      occurs;     // for compound items
      int      position;   // Where in IMAGE buffer this item begins
      boolean  valid[];    // indicates whether type conversion is done

      /** Return the value of this item as an Object. 
        * If this item is compound, the value of element 0 is 
        * returned.  */
      public Object getValue()      { return value[0]; }

      /** Return the value of one element of a compount object   
        * @param index   The index into the compound item
        */
      public Object getValue(int index) { return value[index]; }

      /** Return the type of this object.   See the IMAGE_TYPE
        * constants defined in class Entry */
      public int    getType()       { return type;     }

      /** Return the length of this data item, in bytes.
        * If this is a compound item, the length of one
        * element is returned */
      public int    getLength()     { return length;   }

      /** Returns the number of subitems that make up this item */
      public int    getSubitemCount() { return occurs;   }

      /** Returns the byte offset at which this item begins in
        * the buffer passed to or returned from TurboIMAGE calls */
      public int    getPosition()   { return position; }

      public Object clone() {
         Item o = null;
         Method clone = null;
         try {
            o = (Item) super.clone();
            // Each Object in the value array must be cloned also
            for (int i=0; i<occurs; i++) {
               clone = value[i].getClass().getMethod("clone", null);
               o.value[i] = clone.invoke(value[i], null);
            }
         } catch (Exception e) {
            System.out.println("Item can't be cloned");
         }
         return o;
   }
}

