//: ItemInfo.java

package com.hp.turboimage;
 
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.RemoteException;

/** Class used to return information from DBINFO mode 102
  * (Item Info.  Using the class is a two-step process:
  * First, create an ItemInfo object and call one of its
  * getInfo variants.  Then, the other methods can be used
  * to access the data fields.
  *
  * @version 2.0 (RMI enabled) January 1998
  * @author Mike Yawn
  */
public class ItemInfo extends DatabaseInfoImpl {

   /** Return information about a data item
     * @param base   A Database object
     * @param item   A numeric item number key
     * @param status A DBStatus object
     */
   public void getInfo(Database base, int item, DBStatus status) {
      byte[] name = null;
      try {
         name = base.getName();
      } catch (RemoteException e) {
         e.printStackTrace();
      }
      DBINFO(name, (Object) new Short((short) Math.abs(item)), 
             102, status, this.buffer);
   }

   /** Return information about a data item
     * @param base   A Database object
     * @param item   A String containing the item name
     * @param status A DBStatus object
     */
   public void getInfo(Database base, String item, DBStatus status) {
      byte[] name = null;
      StringBuffer localitem = new StringBuffer(item);
      try {
         name = base.getName();
      } catch (RemoteException e) {
         e.printStackTrace();
      }
      if ((item.charAt(item.length()-1)!=';') && 
          (item.charAt(item.length()-1)!=' ')) {
          localitem.append(';');
      }
      DBINFO(name, (Object) localitem.toString(), 102, status, this.buffer);
   }

   /** Return the name of the data item */
   public String getItemName() {
      // return buffer[0-15] as String
      byte[] b = new byte[16];
      System.arraycopy(buffer, 0, b, 0, 16);
      return new String(b);
   }

   /** Return a String of the item's data type. */
   public String getItemType() {
      // return buffer[16-17] as String
      byte[] b2 = new byte[2];
      System.arraycopy(buffer, 16, b2, 0, 2);
      return new String(b2);
   }

   /** Return the length of each subitem in this item */
   public int getSubitemLength() {
      // return buffer[18-19] as short
      short s;
      byte[] b2 = new byte[2];
      System.arraycopy(buffer, 18, b2, 0, 2);
      s = TypeConverter.bytesToShort(b2);
      // Force to treat as unsigned
      if (s >= 0) return s;
         else return 256 + s;
   }

   /** Return the number of subitems in this item */
   public int getSubitemCount() {
      // return buffer[20-21] as short
      short s;
      byte[] b2 = new byte[2];
      System.arraycopy(buffer, 20, b2, 0, 2);
      s = TypeConverter.bytesToShort(b2);
      // Force to treat as unsigned
      if (s >= 0) return s;
         else return 256 + s;
   }

   public static void main(String[] args) {
      // Test this class
   
      BufferedReader stdin = new BufferedReader(
                             new InputStreamReader(System.in));
      String testname, testpass, testitem;
      Database db;
      DBStatus status = new DBStatus();
      ItemInfo di     = new ItemInfo();

      try {
      System.out.print("Enter database name: ");
      testname = stdin.readLine();
      db = new DatabaseImpl(testname.toUpperCase());
      System.out.print("Enter password: ");
      testpass = stdin.readLine();
      db.setPassword(testpass);
      try {
         db.openSharedRead();
      } catch (IMAGEException e) {
         System.out.println("failed to open " + e);
      }
      System.out.print("Enter item name: ");
      testitem = stdin.readLine() + ";";
      di.getInfo(db, testitem.toUpperCase(), status);
      System.out.println("Item Name: " + di.getItemName());
      System.out.println("Item type: " + di.getItemType());
      System.out.println("Subitem Length: " + di.getSubitemLength());
      System.out.println("Subitem Count:  " + di.getSubitemCount());
      } catch (Exception x) { System.out.println(x); }
   }
}

