//: ItemList.java

package com.hp.turboimage;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.RemoteException;

/** Class used to return information from DBINFO modes 103 & 104
  * (Item Info).  To use this class, create an object of class
  * ItemList and call one of the getInfo variants.  Then call
  * any other methods of interest. 
  *
  * @version 2.0 (RMI enabled) January 1998
  * @author Mike Yawn
  */
public class ItemList extends DatabaseInfoImpl {

   /** Return a list of items in the database 
     * @param base   A Database object
     * @param status A DBStatus object
     */
   public void getInfo(Database base, DBStatus status) {
      byte[] name = null;
      try {
         name = base.getName();
      } catch (RemoteException e) {
         e.printStackTrace();
      }
      DBINFO(name, (Object) new Short((short)0), 103, 
             status, this.buffer);
   }

   /** Return a list of items in a dataset 
     * @param base   A Database object
     * @param set    A dataset number 
     * @param status A DBStatus object
     */
   public void getInfo(Database base, int set, DBStatus status) {
      byte[] name = null;
      try {
         name = base.getName();
      } catch (RemoteException e) {
         e.printStackTrace();
      }
      DBINFO(name, (Object) new Short((short)set), 104, 
             status, this.buffer);
   }
   /** Return a list of items in a dataset 
     * @param base   A Database object
     * @param item   A String containing a dataset name
     * @param status A DBStatus object
     */
   public void getInfo(Database base, String set, DBStatus status) {
      byte[] name = null;
      StringBuffer localset = new StringBuffer(set);
      try {
         name = base.getName();
      } catch (RemoteException e) {
         e.printStackTrace();
      }
      if ((set.charAt(set.length()-1)!=';') &&
          (set.charAt(set.length()-1)!=' ')) {
          localset.append(';');
      }
      DBINFO(name, (Object) localset.toString(), 104, status, this.buffer);
   }

   /** Return the number of items in the itemlist */
   public int getItemCount() {
      // return buffer[0-1] as short 
      byte[] ba = new byte[2];
      System.arraycopy(buffer, 0, ba, 0, 2);
      return TypeConverter.bytesToShort(ba);
   }

   /** Return an itemnumber out of the item list 
     * @param index  The position of the desired itemnumber
     */
   public int getItemNumber(int index) {
      // return buffer[index*2 - index*2+1] as short
      byte[] ba = new byte[2];
      System.arraycopy(buffer, index*2, ba, 0, 2);
      return TypeConverter.bytesToShort(ba);
   }

   /** Determine whether the requested item is readable
     * @param index  The position of the desired itemnumber */
   public boolean isReadable(int index) { return true; }

   /** Determine whether the requested item is writeable
     * @param index  The position of the desired itemnumber */
   public boolean isWriteable(int index) { return getItemNumber(index) < 0; }

   public static void main(String[] args) {
      // Test this class
   
      BufferedReader stdin = new BufferedReader(
                             new InputStreamReader(System.in));
      String testname, testpass, testitem;
      Database db;
      DBStatus status = new DBStatus();
      ItemList di     = new ItemList();
      int count;
      int item;

      try {
      System.out.print("Enter database name: ");
      testname = stdin.readLine();
      db = new DatabaseImpl(testname.toUpperCase());
      System.out.print("Enter password: ");
      testpass = stdin.readLine();
      db.setPassword(testpass);
      try {
         db.openSharedModify();
      } catch (IMAGEException e) {
         System.out.println("failed to open " + e);
      }

      di.getInfo(db, status);
      count = di.getItemCount();
      System.out.println("Item Count: " + count);
      for (int i=1; i<=count; i++) {
         item = di.getItemNumber(i);
         System.out.print("Item Number: " + item  + " Access: ");
         if (di.isReadable(i)) {
            System.out.print("  READ");
          }
         if (di.isWriteable(i)) {
            System.out.print("  WRITE");
         }
         System.out.println();
      }
      } catch (Exception x) { System.out.println(x); }
   }
}

