//: ServerImpl.java

package com.hp.turboimage;
 
import java.rmi.*;
import java.rmi.server.*;

/** This class is the RMI server for TurboIMAGE 
 *  databases.
 *
 * @version 2.0 (RMI enabled) January 1998
 * @author  Mike Yawn
 */
public class ServerImpl extends UnicastRemoteObject
                     implements TurboIMAGEServer {

   public ServerImpl() throws RemoteException {};

   public Database getDatabase(String dbRootFile) throws RemoteException {
      Database db = new DatabaseImpl(dbRootFile);
      return db;
   }

   public Dataset getDataset(Database db, String setname)
                                          throws RemoteException {
      Dataset ds = new DatasetImpl(db, setname);
      return ds;
   }

   public Entry getEntry(Database db, Dataset ds) throws RemoteException {
      Entry  ent = new EntryImpl(db, ds);
      return ent;
   }

   private static void usage() {
      System.out.println("Usage: java com.hp.turboimage.Server [options]");
      System.out.println();
      System.out.println("where options include: ");
      System.out.println("   -registry     start RMI registry");
      System.exit(0);
   } 

   private static boolean startRegistry(int port) {
      if (port == 0) port = 1099;
      try {
         java.rmi.registry.LocateRegistry.createRegistry(port);
         return true;
      } catch (java.rmi.RemoteException e) {
        e.printStackTrace();
        return false;
      }
   }

   private static boolean startServer() {
      try {
         TurboIMAGEServer s = new ServerImpl();
         String hostname = java.net.InetAddress.getLocalHost().getHostName();
         java.rmi.Naming.rebind("//" + hostname + 
                                 "/TurboIMAGEFactory", s);
         System.out.println("TurboIMAGEFactory running on " + hostname);
         return true;
      } catch (java.rmi.ConnectException ce) {
         System.out.println("Registry not running.  Either start a ");
         System.out.println("separate registry job, or use the -registry");
         System.out.println("command line option");
         return false;
      } catch (Exception e) {
         e.printStackTrace();
         return false;
      }
   }

   public static void main(String[] argv) {
      boolean startRegistry = false;
      boolean logging       = false;
      int     port = 0;

      for (int i=0; i<argv.length; i++) {
// REMIND: should look for, parse, and pass a port number
         if (argv[i].equals("-registry")) { 
            startRegistry = true; 
            continue; 
         }
         if (argv[i].equals("-logging")) {
            logging = true;
            continue;
         }
         // unrecognized command line option.
         usage();
      }

      // Setup properties
      java.util.Properties p;
      p = System.getProperties();
// REMIND: codebase should not be a hard-coded value
      p.put("java.rmi.server.codebase",
            "http://belle.cup.hp.com//JAVA/DEMO/");
      if (logging) {
         //p.put("sun.rmi.transport.tcp.logLevel","verbose");
         //p.put("sun.rmi.transport.logLevel","verbose");
         p.put("sun.rmi.server.logLevel","verbose");
      }
      System.setProperties(p);

      // Start the security manager 
      System.setSecurityManager(new RMISecurityManager());

      // Start the registry, if requested
      if (startRegistry) {
         if (startRegistry(port)) 
            System.out.println("Registry started.");
         else                 
            System.out.println("Failed to start registry.");
      }

      startServer();
   }

}

