//: SetInfo.java

package com.hp.turboimage;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/** Class used to return information from DBINFO mode 202
  * (Item Info).  To use this class, create an object of
  * class SetInfo and call one of the getInfo variants.  
  * Then use the remaining calls to access items of interest.
  *
  * @version 2.0 (RMI enabled) January 1998
  * @author Mike Yawn
  */

public class SetInfo extends DatabaseInfoImpl {

   /** Return information about a data item
     * @param base   A Database object
     * @param item   A numeric item number key
     * @param status A DBStatus object
     */
   public void getInfo(Database base, int item, DBStatus status)
                       throws java.rmi.RemoteException  {
      DBINFO(base.getName(), (Object) new Short((short) item), 202, 
                  status, this.buffer);
   }

   /** Return information about a data item
     * @param base   A Database object
     * @param item   A String containing the item name
     * @param status A DBStatus object
     */
   public void getInfo(Database base, String item, DBStatus status) 
                       throws java.rmi.RemoteException {
      DBINFO(base.getName(), (Object) item, 202, status, this.buffer);
   }

   /** Return the name of the data set */
   public String getSetName() {
      // return buffer[0-15] as String
      byte[] b = new byte[16];
      System.arraycopy(buffer, 0, b, 0, 16);
      return new String(b);
   }

   /** Return a two-byte string representing the set type
     * ("A " - automatic master, "M " - manual master, 
     *  "D " - detail) */
   public String getSetType() {
      // return buffer[16-17] as String
      byte[] b = new byte[2];
      System.arraycopy(buffer, 16, b, 0, 2);
      return new String(b);
   }

   /** Return the length of entries in this data set */
   public int getEntryLength() {
      // return buffer[18-19] as short
      byte[] b = new byte[2];
      System.arraycopy(buffer, 18, b, 0, 2);
      return TypeConverter.bytesToShort(b);
   }

   /** Return the blocking factor used for this data set */
   public int getBlockingFactor() {
      // return buffer[20-21] as short
      byte[] b = new byte[2];
      System.arraycopy(buffer, 20, b, 0, 2);
      return TypeConverter.bytesToShort(b);
   }

   /** Return the current number of entries in this data set */
   public int getNumEntries() {
      // return buffer[26-29] as int
      byte[] b = new byte[4];
      System.arraycopy(buffer, 26, b, 0, 4);
      return TypeConverter.bytesToInt(b);
   }

   /** Return the capacity of this data set */
   public int getCapacity() {
      // return buffer[30-33] as int
      byte[] b = new byte[4];
      System.arraycopy(buffer, 30, b, 0, 4);
      return TypeConverter.bytesToInt(b);
   }

   public static void main(String[] args) {
      // Test this class
   
      BufferedReader stdin = new BufferedReader(
                             new InputStreamReader(System.in));
      String testname, testpass, testset;
      Database db;
      DBStatus status = new DBStatus();
      SetInfo  di     = new SetInfo();

      try {
      System.out.print("Enter database name: ");
      testname = stdin.readLine();
      db = new DatabaseImpl(testname.toUpperCase());
      System.out.print("Enter password: ");
      testpass = stdin.readLine();
      db.setPassword(testpass);
      try {
         db.openSharedRead();
      } catch (IMAGEException e) {
         System.out.println("failed to open " + e);
      }
      System.out.print("Enter set name: ");
      testset = stdin.readLine() + ";";
      di.getInfo(db, testset.toUpperCase(), status);
      System.out.println("Set Name: " + di.getSetName());
      System.out.println("Set Type: " + di.getSetType());
      System.out.println("Entry Length: " + di.getEntryLength());
      System.out.println("Blocking Factor:  " + di.getBlockingFactor());
      System.out.println("Number Entries: " + di.getNumEntries());
      System.out.println("Capacity: " + di.getCapacity());
      } catch (Exception x) { System.out.println(x); }
   }
}

