//: SetList.java

package com.hp.turboimage;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.rmi.RemoteException;

/** Class used to return information from DBINFO modes 203 & 204
  * (Set Info) 
  *
  * @version 2.0 (RMI enabled) January 1998
  * @author Mike Yawn
  */

public class SetList extends DatabaseInfoImpl {

// First, must call a getInfo variant to do the DBINFO

   /** When called with only a basename and status, getInfo
     * will return information about all sets in the database.
     * @param base A database object that has been opened
     * @param status a DBStatus object
     */
   public void getInfo(Database base, DBStatus status) 
                       throws java.rmi.RemoteException {
      DBINFO(base.getName(), (Object) new Short((short)0), 203, 
             status, this.buffer);
   }

   /** When called with a set identifier, getInfo returns
     * information about the requested set.  This variant
     * takes a numeric set identifier
     * @param base  A Database object
     * @param item  An item number
     * @param status A DBStatus object
     */
   public void getInfo(Database base, int item, DBStatus status) 
                       throws java.rmi.RemoteException {
      DBINFO(base.getName(), (Object) new Short((short)item), 204, 
             status, this.buffer);
   }

   /** This is another variant of getInfo for returning information
     * about a specific dataset.  In this version, the set name is
     * used as the key.
     * @param base  A Database object
     * @param item  An Item name
     * @param status A DBStatus object
     */
   public void getInfo(Database base, String item, DBStatus status) 
                       throws java.rmi.RemoteException {
      byte[] name = null;
      StringBuffer localitem = new StringBuffer(item);
      try {
         name = base.getName();
      } catch (RemoteException e) {
         e.printStackTrace();
      }
      if ((item.charAt(item.length()-1)!=';') &&
          (item.charAt(item.length()-1)!=' ')) {
          localitem.append(';');
      }
      DBINFO(name, (Object) localitem.toString(), 204, status, this.buffer);
   }

   /** Returns the number of sets for which information was returned
     * by the most recent SetList.getInfo call */
   public int getSetCount() {
      // return buffer[0-1] as short 
      byte[] b = new byte[2];
      System.arraycopy(buffer, 0, b, 0, 2);
      return TypeConverter.bytesToShort(b);
   }

   /** Returns the set id number for the requested set 
     * @param index  Index into the list of sets returned by 
     * the most recent SetList.getInfo call
     */
   public int getSetNumber(int index) {
      // return buffer[index*2 - index*2+1] as short
      byte[] b = new byte[2];
      System.arraycopy(buffer, index*2, b, 0, 2);
      return TypeConverter.bytesToShort(b);

   }

   /** Returns true if the requested set is readable by the
     * requestor  */
      // note: always true; it wouldn't be returned to us otherwise 
   public boolean isReadable(int index) { return true; }

   /** Returns true if the requested set is writeable by the
     * requestor */
   public boolean isWriteable(int index) { return getSetNumber(index) < 0; }

   public static void main(String[] args) {
      // Test this class
   
      BufferedReader stdin = new BufferedReader(
                             new InputStreamReader(System.in));
      String testname, testpass, testitem;
      Database db;
      DBStatus status = new DBStatus();
      SetList di      = new SetList();
      int count;
      int set;

      try {
      System.out.print("Enter database name: ");
      testname = stdin.readLine();
      db = new DatabaseImpl(testname.toUpperCase());
      System.out.print("Enter password: ");
      testpass = stdin.readLine();
      db.setPassword(testpass);
      try {
         db.openSharedModify();
      } catch (IMAGEException e) {
         System.out.println("failed to open " + e);
      }

      di.getInfo(db, status);
      count = di.getSetCount();
      System.out.println("Set Count: " + count);
      for (int i=1; i<=count; i++) {
         set = di.getSetNumber(i);
         System.out.print("Set Number: " + set  + " Access: ");
         if (di.isReadable(i)) {
            System.out.print("  READ");
          }
         if (di.isWriteable(i)) {
            System.out.print("  WRITE");
         }
         System.out.println();
      }
      } catch (Exception x) { System.out.println(x); }
   }
}

