//: TypeConverterImpl.c
#include "com_hp_turboimage_TypeConverter.h"

typedef union { 
    char    asBytes[2];
    short   asShort;
} ByteArray2;

typedef union {
    char    asBytes[4];
    int     asInteger;
    float   asFloat;
} ByteArray4;

typedef union {
    char        asBytes[8];
    long long   asLongLong;  
    double      asDouble;
} ByteArray8;

typedef union {
    char    asByte;
    struct {
       unsigned int n1:4;
       unsigned int n2:4;
    } asNibbles;
} Byte;

int i;

// *** HPFPCONVERT prototype and constants
extern void HPFPCONVERT(int parms, void *source, void *dest,
            short sformat, short dformat, int *status, 
            short *except, short roundmode);

   // Data formats
   const short FMT_HP3K_32 = 1;
   const short FMT_HP3K_64 = 2;
   const short FMT_IEEE_32 = 3;
   const short FMT_IEEE_64 = 4;
   const short FMT_IEEE_128= 5;

   // Exceptions
   const short EXC_INEXACT           = 1;
   const short EXC_UNDERFLOW_INEXACT = 3;
   const short EXC_OVERFLOW_INEXACT  = 5;
   const short EXC_INVALID_OP        = 8;

   // Rounding Modes
   const short RM_NEAREST      = 0;
   const short RM_ZERO         = 1;
   const short RM_POS_INFINITY = 2;
   const short RM_NEG_INFINITY = 3;

// bytesToShort
JNIEXPORT jshort JNICALL 
Java_com_hp_turboimage_TypeConverter_bytesToShort(JNIEnv    *env, 
                                                  jclass     this, 
                                                  jbyteArray in) {
   ByteArray2 conv;
   jbyte *input = (*env)->GetByteArrayElements(env, in, 0);
   for(i=0; i<2; i++)
      conv.asBytes[i] = input[i];
   (*env)->ReleaseByteArrayElements(env, in, input, 0);
   return conv.asShort;
}

// bytesToInt
JNIEXPORT jint JNICALL 
Java_com_hp_turboimage_TypeConverter_bytesToInt(JNIEnv    *env, 
                                                jclass     this, 
                                                jbyteArray in) {
   ByteArray4 conv;
   jbyte *input = (*env)->GetByteArrayElements(env, in, 0);
   for(i=0; i<4; i++)
      conv.asBytes[i] = input[i];
   (*env)->ReleaseByteArrayElements(env, in, input, 0);
   return conv.asInteger;
}

// bytesToLong
JNIEXPORT jlong JNICALL 
Java_com_hp_turboimage_TypeConverter_bytesToLong(JNIEnv    *env, 
                                                 jclass     this, 
                                                 jbyteArray in) {
   ByteArray8 conv;
   jbyte *input = (*env)->GetByteArrayElements(env, in, 0);
   for(i=0; i<8; i++)
      conv.asBytes[i] = input[i];
   (*env)->ReleaseByteArrayElements(env, in, input, 0);
   return conv.asLongLong; 
}

// packedBytesToString
JNIEXPORT jstring JNICALL
Java_com_hp_turboimage_TypeConverter_packedBytesToString(JNIEnv    *env,
                                                         jclass     this,
                                                         jbyteArray in) {
   int j=0;
   char *work;
   jstring output;
   Byte conv;
   jbyte *input = (*env)->GetByteArrayElements(env, in, 0);
   jint blength = (*env)->GetArrayLength(env, in);
   jint slength = blength*2;

   work = (void *) malloc(slength+1); // 1 for terminator
   work[j++] = '0';  // placeholder for sign
   for (i=0; i<blength; i++) {
      conv.asByte = input[i];
      work[j++] = conv.asNibbles.n1 + 48;
      if (i < (blength-1)) {
         work[j++] = conv.asNibbles.n2 + 48; 
      }
   }
   work[j] = NULL; // terminate string
   while(*work == '0') work++; // skip leading zeroes
   if (conv.asNibbles.n2 == 12) *--work = '+'; // back up & add sign
   if (conv.asNibbles.n2 == 13) *--work = '-';
   output = (*env)->NewStringUTF(env, work);
   (*env)->ReleaseByteArrayElements(env, in, input, 0);
   return output;
}

// bytesToFloat
JNIEXPORT jfloat JNICALL 
Java_com_hp_turboimage_TypeConverter_bytesToFloat(JNIEnv    *env, 
                                                  jclass     this, 
                                                  jbyteArray in, 
                                                  jint       format) {
   ByteArray4 conv;
   jbyte *input = (*env)->GetByteArrayElements(env, in, 0);
   for(i=0; i<4; i++) 
      conv.asBytes[i] = input[i];
   if (format == 2) {
      float source = conv.asFloat;
      int status;
      short  exception;
      HPFPCONVERT(7, &source, &conv.asFloat, FMT_HP3K_32, FMT_IEEE_32,
         &status, &exception, RM_NEAREST);
      if (status != 0) printf("HPFPCONVERT float, 3K->IEEE failed\n");
   }
   (*env)->ReleaseByteArrayElements(env, in, input, 0);
   return conv.asFloat;
}

// bytesToDouble
JNIEXPORT jdouble JNICALL 
Java_com_hp_turboimage_TypeConverter_bytesToDouble(JNIEnv    *env, 
                                                   jclass     this, 
                                                   jbyteArray in, 
                                                   jint       format) {
   ByteArray8 conv;
   jbyte *input = (*env)->GetByteArrayElements(env, in, 0);
   for(i=0; i<8; i++) {
      conv.asBytes[i] = input[i];
   }
   if (format == 2) {
      double source = conv.asDouble;
      int status;
      short exception;
      HPFPCONVERT(7, &source, &conv.asDouble, FMT_HP3K_64, FMT_IEEE_64,
         &status, &exception, RM_NEAREST);
      if (status != 0) printf ("HPFPCONVERT double, 3K->IEEE failed\n");
   }
   (*env)->ReleaseByteArrayElements(env, in, input, 0);
   return conv.asDouble;

}

// shortToBytes
JNIEXPORT jbyteArray JNICALL 
Java_com_hp_turboimage_TypeConverter_shortToBytes(JNIEnv    *env, 
                                                  jclass     this, 
                                                  jshort     in ) {
   ByteArray2 conv;
   jbyteArray ref    = (*env)->NewByteArray(env, 2);
   jbyte     *output = (*env)->GetByteArrayElements(env, ref, 0); 
   conv.asShort = in;
   for(i=0; i<2; i++)
      output[i] = conv.asBytes[i];
   (*env)->ReleaseByteArrayElements(env, ref, output, 0);
   return ref;

}

// intToBytes
JNIEXPORT jbyteArray JNICALL 
Java_com_hp_turboimage_TypeConverter_intToBytes(JNIEnv    *env, 
                                                jclass     this, 
                                                jint       in ) {
   ByteArray4 conv;
   jbyteArray ref    = (*env)->NewByteArray(env, 4);
   jbyte     *output = (*env)->GetByteArrayElements(env, ref, 0); 
   conv.asInteger = in;
   for(i=0; i<4; i++)
      output[i] = conv.asBytes[i];
   (*env)->ReleaseByteArrayElements(env, ref, output, 0);
   return ref;
}

// longToBytes
JNIEXPORT jbyteArray JNICALL 
Java_com_hp_turboimage_TypeConverter_longToBytes(JNIEnv    *env, 
                                                 jclass    this, 
                                                 jlong      in ) {
   ByteArray8 conv;
   jsize len; // just for testing
   jbyteArray ref    = (*env)->NewByteArray(env, 8);
   jbyte     *output = (*env)->GetByteArrayElements(env, ref, 0);
   conv.asLongLong = in;
   for(i=0; i<8; i++)
      output[i] = conv.asBytes[i];
   (*env)->ReleaseByteArrayElements(env, ref, output, 0);
   return ref;

}

// stringToPackedBytes
JNIEXPORT jbyteArray JNICALL
Java_com_hp_turboimage_TypeConverter_stringToPackedBytes(JNIEnv *env,
                                                         jclass  this,
                                                         jstring in,
                                                         jint    blength ) {
   int sign = 0;     // 0 - unsigned, 1 - positive, -1 - negative
   int left = 0;     // 1 when next digit goes in left half of byte
   int outbyte = blength-1; // index into output array
   Byte conv;

   jbyteArray    ref = (*env)->NewByteArray(env, blength);
   jbyte     *output = (*env)->GetByteArrayElements(env, ref, 0);
   const char *input = (*env)->GetStringUTFChars(env, in, 0);
   jint      slength = (*env)->GetStringLength(env, in);

   // Reserve space for sign in right half of last byte
   left = 1;
   // Walk the string, transferring digits into byte array
   for (i=slength-1; i>=0; i--) {
      // test for sign
      if (input[i] == '+') { sign =  1; continue; }
      if (input[i] == '-') { sign = -1; continue; }
      // test for ignored characters
      if ((input[i] == ',' ) || (input[i] == '.')) continue;
      // test for digit
      if ((input[i] >= '0') && (input[i] <= '9')) {
         if (left) {
             conv.asNibbles.n1 = input[i];
             output[outbyte--] = conv.asByte;
             left = 0;
         } else {
             if (outbyte < 0) {
                printf("overflowed output buffer!\n");
                break;
             }
             conv.asNibbles.n2 = input[i];
             left = 1;
         }
         continue;
      }
      // error: unexpected character
      printf("unexpected character %c in string\n", input[i]);
   }
   // complete last byte if only half-full 
   if (left == 1) {
      conv.asNibbles.n1 = 0;
      output[outbyte--] = conv.asByte;
   }
   while (outbyte >= 0) {
      // zero-fill any unused bytes
      conv.asNibbles.n1 = 0;
      conv.asNibbles.n2 = 0;
      output[outbyte--] = conv.asByte;
   }
   // set sign byte
   conv.asByte = output[blength-1];
   if (sign == 0) conv.asNibbles.n2 = 15;
   else if (sign == 1) conv.asNibbles.n2 = 12;
   else conv.asNibbles.n2 = 13;
   output[blength-1] = conv.asByte;

   // clean up and return
   (*env)->ReleaseByteArrayElements(env, ref, output, 0);
   (*env)->ReleaseStringUTFChars(env, in, input);
   return ref;
}

// floatToBytes
JNIEXPORT jbyteArray JNICALL 
Java_com_hp_turboimage_TypeConverter_floatToBytes(JNIEnv    *env, 
                                                  jclass     this, 
                                                  jfloat     in, 
                                                  jint       format) {
   ByteArray4 conv;
   jbyteArray ref    = (*env)->NewByteArray(env, 4);
   jbyte     *output = (*env)->GetByteArrayElements(env, ref, 0); 
   if (format == 2) {
      float source = in;
      int status;
      short exception;
      HPFPCONVERT(7, &source, &in, FMT_IEEE_32, FMT_HP3K_32,
         &status, &exception, RM_NEAREST);
      if (status != 0) printf ("HPFPCONVERT float, IEEE->3K failed\n");
   }
   conv.asFloat = in;
   for(i=0; i<4; i++)
      output[i] = conv.asBytes[i];
   (*env)->ReleaseByteArrayElements(env, ref, output, 0);
   return ref;
}

// doubleToBytes
JNIEXPORT jbyteArray JNICALL 
Java_com_hp_turboimage_TypeConverter_doubleToBytes(JNIEnv    *env, 
                                                   jclass     this, 
                                                   jdouble    in, 
                                                   jint       format) {
   ByteArray8 conv;
   jbyteArray ref    = (*env)->NewByteArray(env, 8);
   jbyte     *output = (*env)->GetByteArrayElements(env, ref, 0); 
   if (format == 2) {
      double source = in;
      int status;
      short exception;
      HPFPCONVERT(7, &source, &in, FMT_IEEE_64, FMT_HP3K_64, 
         &status, &exception, RM_NEAREST);
      if (status != 0) printf("HPFPCONVERT double, IEEE->3K failed\n");
   }
   conv.asDouble = in;
   for(i=0; i<8; i++) 
      output[i] = conv.asBytes[i];
   (*env)->ReleaseByteArrayElements(env, ref, output, 0);
   return ref;
}
