//: GenericClient.java

import com.hp.turboimage.*;

import java.io.*;
import java.net.*;

/** This class provides an example of how to call the 
 *  Java TurboIMAGE library routines.  This class will open
 *  a database and read a dataset serially, displaying each 
 *  record.  At this point, only those functions required to
 *  support this program are actually implemented by the
 *  TurboIMAGE library.<p>
 *
 * <a href=DBClient.java>View the source code for this class</a><p>
 *
 * @author Mike Yawn
 * @version prerelease, October 1996
 */
public class GenericClient {

   public static void main(String argv[]) {

      Database empDB       = null;
      Dataset  empDetail   = null;
      Entry    detailEntry = null;
      String root;

      try {
         // Open a database
         empDB = new DatabaseImpl("EMPDB.IMAGE.JAVA");
         empDB.setPassword("PASSWORD;");
         empDB.openSharedRead();
                           
         // Create a dataset  
         empDetail = new DatasetImpl(empDB, "EMPDTL");

         //Create an Entry for the dataset 
         detailEntry = new EntryImpl(empDB, empDetail);
         detailEntry.allElements();

         // Read the dataset and display results 
         while(empDetail.readSerial(detailEntry)){
            System.out.println(detailEntry.getValue("EMPLOYEE-NUMBER") 
               + " " + detailEntry.getValue("FIRST-NAME")
               + " " + detailEntry.getValue("LAST-NAME")       );
         }
         System.out.println("No more entries in dataset.");
         System.out.println("Normal termination.");
         System.exit(0);
      } catch (IMAGEException e2) {
         System.out.println(e2.toString());
      } catch (java.rmi.RemoteException e3) {
         e3.printStackTrace();
         System.out.println(e3.toString());
      } catch (Exception e4) {
         System.out.println(e4);
      } finally {
         if (empDB != null)
            try {
               empDB.close();
            } catch (Exception e) {}
         System.exit(0);
      }
   }
}
